﻿using InternetShopContracts.DataBindingModels;
using InternetShopContracts.DataSearchModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.StorageContracts;
using InternetShopDatabase.Models;

namespace InternetShopDatabase.Storages
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? Delete(OrderSearchModel model)
        {
            using var context = new InternetShopDatabase();
            var item = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (item != null)
            {
                context.Orders.Remove(item);
                context.SaveChanges();
                return item.GetViewModel;
            }
            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            return GetFilteredList(model).FirstOrDefault();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            var elements = GetFullList();
            if (model.Id != null)
            {
                elements = elements.Where(x => x.Id == model.Id).ToList();
            }
            if (model.CustomerFIO != null)
            {
                elements = elements.Where(x => x.CustomerFIO.Contains(model.CustomerFIO)).ToList();
            }
            if (model.CustomerEmail != null)
            {
                elements = elements.Where(x => x.CustomerEmail.Contains(model.CustomerEmail)).ToList();
            }
            return elements;
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new InternetShopDatabase();
            return context.Orders.Select(x => x.GetViewModel).ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new InternetShopDatabase();
            var item = Order.Create(model);
            context.Orders.Add(item);
            context.SaveChanges();
            return item.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new InternetShopDatabase();
            var item = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (item == null)
            {
                return null;
            }
            item.Update(model);
            context.SaveChanges();
            return item.GetViewModel;
        }
    }
}
