﻿using InternetShopContracts.DataBindingModels;
using InternetShopContracts.DataSearchModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.StorageContracts;
using InternetShopDatabase.Models;

namespace InternetShopDatabase.Storages
{
    public class ProductStorage : IProductStorage
    {
        public ProductViewModel? Delete(ProductSearchModel model)
        {
            using var context = new InternetShopDatabase();
            var item = context.Products.FirstOrDefault(rec => rec.Id == model.Id);
            if (item != null)
            {
                context.Products.Remove(item);
                context.SaveChanges();
                return item.GetViewModel;
            }
            return null;
        }

        public ProductViewModel? GetElement(ProductSearchModel model)
        {
            return GetFilteredList(model).FirstOrDefault();
        }

        public List<ProductViewModel> GetFilteredList(ProductSearchModel model)
        {
            var elements = GetFullList();
            if (model.Id != null)
            {
                elements = elements.Where(x => x.Id == model.Id).ToList();
            }
            if (model.Name != null)
            {
                elements = elements.Where(x => x.Name.Contains(model.Name)).ToList();
            }
            return elements;
        }

        public List<ProductViewModel> GetFullList()
        {
            using var context = new InternetShopDatabase();
            return context.Products.Select(x => x.GetViewModel).ToList();
        }

        public ProductViewModel Insert(ProductBindingModel model)
        {
            using var context = new InternetShopDatabase();
            var item = Product.Create(model);
            context.Products.Add(item);
            context.SaveChanges();
            return item.GetViewModel;
        }

        public ProductViewModel? Update(ProductBindingModel model)
        {
            using var context = new InternetShopDatabase();
            var item = context.Products.FirstOrDefault(x => x.Id == model.Id);
            if (item == null)
            {
                return null;
            }
            item.Update(model);
            context.SaveChanges();
            return item.GetViewModel;
        }
    }
}
