﻿using InternetShopContracts.DataBindingModels;
using InternetShopContracts.DataSearchModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.LogicsContracts;
using InternetShopContracts.StorageContracts;

namespace InternetShopLogics.Logics
{
    public class ProductLogic : IProductLogic
    {
        private IProductStorage _productStorage;

        public ProductLogic(IProductStorage productStorage)
        {
            _productStorage = productStorage;
        }

        public bool Create(ProductBindingModel model)
        {
            CheckModel(model);
            if (_productStorage.Insert(model) == null)
            {
                return false;
            }
            return true;
        }

        public bool Delete(ProductSearchModel model)
        {
            if (_productStorage.Delete(model) == null)
            {
                return false;
            }
            return true;
        }

        public ProductViewModel? ReadElement(ProductSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            return _productStorage.GetElement(model);
        }

        public List<ProductViewModel> ReadList(ProductSearchModel? model = null)
        {
            List<ProductViewModel>? list = null;
            if (model == null)
            {
                list = _productStorage.GetFullList();
            }
            else
            {
                list = _productStorage.GetFilteredList(model);
            }
            if (list == null)
            {
                return new List<ProductViewModel>();
            }
            return list;
        }

        public bool Update(ProductBindingModel model)
        {
            CheckModel(model);
            if (_productStorage.Update(model) == null)
            {
                return false;
            }
            return true;
        }

        private void CheckModel(ProductBindingModel model, bool checkParams = true)
        {
            if (model == null) throw new ArgumentNullException(nameof(model));
            if (!checkParams) return;
            if (string.IsNullOrEmpty(model.Name))
            {
                throw new ArgumentNullException("Нет названия товара", nameof(model.Name));
            }
            var item = _productStorage.GetElement(new ProductSearchModel
            {
                Name = model.Name
            });
            if (item != null && item.Id != model.Id)
            {
                throw new InvalidOperationException("Товар с таким названием уже есть");
            }
        }
    }
}
